/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package controller;

import DAO.*;
import Exceptions.InvalidParameter;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import service.User;
import java.util.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;
import org.springframework.web.servlet.mvc.multiaction.MultiActionController;
import javax.servlet.http.*;
import javax.servlet.*;
import org.hibernate.exception.ConstraintViolationException;

/**
 *
 * @author faycal
 */
public class BddController extends MultiActionController {

    private String login;
    private String pwd;
    private User utilisateur;
    private HttpSession session;

    public BddController() {
    }

    private void sessionInit(HttpServletRequest request) {
        if (login == null) {
            login = request.getUserPrincipal().getName();
        }
        if (utilisateur == null) {
            utilisateur = new User(login);
        }

        if (session == null) {
            session = request.getSession();
            session.setAttribute("user", utilisateur);
        }
    }

    public ModelAndView menu(HttpServletRequest request,
            HttpServletResponse response) {
        sessionInit(request);
        System.out.println("nbuser=" + User.getCompteur());
        return new ModelAndView("menu").addObject("user", utilisateur);

    }

    public ModelAndView logout(HttpServletRequest request,
            HttpServletResponse response) {
        request.getSession().invalidate();

        return new ModelAndView("logout");

    }

    public ModelAndView list(HttpServletRequest request,
            HttpServletResponse response) throws Exception {
        sessionInit(request);
        String type = request.getParameter("type");
        ModelAndView mv = new ModelAndView("resultat");
        mv.addObject("user", session.getAttribute("user"));
        mv.addObject("type", type);

        switch (type) {
            case "customer":
                mv.addObject("results", new MagasinHelper().getClientsColums());
                mv.addObject("colonnes", Arrays.asList("customerId", "name", "addressline1", "addressline2", "zip", "rate"));
                break;
            case "product":
                mv.addObject("results", new MagasinHelper().getProductsColums());
                mv.addObject("colonnes", Arrays.asList("productId", "manufacturerId", "productCode", "purchaseCost", "quantityOnHand", "markup", "available", "description"));
                mv.addObject("relation", new HashMap<String, String>() {
                    {
                        put("productCode", "prodCode");
                    }
                });
                break;
            case "purchase":
                mv.addObject("results", new MagasinHelper().getPurchaseColums());
                mv.addObject("colonnes", Arrays.asList("orderNum", "customerId", "productId", "quantity", "shippingCost", "salesDate", "shippingDate", "freightCompany"));
                mv.addObject("relation", new HashMap<String, String>() {
                    {
                        put("customerId", "customer");
                        put("productId", "product");
                    }
                });
                break;
            case "discount":
                mv.addObject("results", new MagasinHelper().getDiscountCodesColums());
                mv.addObject("colonnes", Arrays.asList("discountCode", "rate"));
                break;
            case "prodCode":
                mv.addObject("results", new MagasinHelper().getProductCodesColums());
                mv.addObject("colonnes", Arrays.asList("prodCode", "discountCode", "description"));
                mv.addObject("relation", new HashMap<String, String>() {
                    {
                        put("discountCode", "discount");
                    }
                });
                break;
            default:
                mv.addObject("error", "Type not found");
        }

        return mv;
    }

    public ModelAndView formAdd(HttpServletRequest request,
            HttpServletResponse response) throws Exception {
        sessionInit(request);
        String type = request.getParameter("type");
        ModelAndView mv = new ModelAndView("detail");
        mv.addObject("user", session.getAttribute("user"));
        mv.addObject("type", type);
        mv.addObject("save", true);

        switch (type) {
            case "customer":
                mv.addObject("code", new MagasinHelper().getDiscountCodes());
                mv.addObject("cpostaux", new MagasinHelper().getZipCode());
                break;
            case "product":
                mv.addObject("manufacturers", new MagasinHelper().getManufacturers());
                mv.addObject("productCodes", new MagasinHelper().getProductCodes());
                break;
            case "purchase":
                mv.addObject("customers", new MagasinHelper().getClients());
                mv.addObject("products", new MagasinHelper().getProducts());
                break;
            case "discount":
                break;
            case "prodCode":
                mv.addObject("code", new MagasinHelper().getDiscountCodes());
                break;
            default:
                mv.addObject("error", "Type not found");
        }

        return mv;
    }

    public ModelAndView detail(HttpServletRequest request,
            HttpServletResponse response) throws Exception {
        sessionInit(request);
        String type = request.getParameter("type");
        ModelAndView mv = new ModelAndView("detail");
        mv.addObject("user", session.getAttribute("user"));
        mv.addObject("type", type);

        switch (type) {
            case "customer":
                mv.addObject("result", new MagasinHelper().getClient(Integer.parseInt(request.getParameter("num"))));
                mv.addObject("code", new MagasinHelper().getDiscountCodes());
                mv.addObject("cpostaux", new MagasinHelper().getZipCode());
                break;
            case "product":
                mv.addObject("result", new MagasinHelper().getProduct(Integer.parseInt(request.getParameter("num"))));
                mv.addObject("manufacturers", new MagasinHelper().getManufacturers());
                mv.addObject("productCodes", new MagasinHelper().getProductCodes());
                break;
            case "purchase":
                mv.addObject("result", new MagasinHelper().getPurchase(Integer.parseInt(request.getParameter("num"))));
                mv.addObject("customers", new MagasinHelper().getClients());
                mv.addObject("products", new MagasinHelper().getProducts());
                break;
            case "discount":
                mv.addObject("result", new MagasinHelper().getDiscountCode(request.getParameter("num").charAt(0)));
                break;
            case "prodCode":
                mv.addObject("result", new MagasinHelper().getProductCode(request.getParameter("num")));
                mv.addObject("code", new MagasinHelper().getDiscountCodes());
                break;
            default:
                mv.addObject("error", "Type not found");
        }

        return mv;
    }

    public ModelAndView find(HttpServletRequest request,
            HttpServletResponse response) throws Exception {
        sessionInit(request);
        String type = request.getParameter("type");
        ModelAndView mv = new ModelAndView("resultat");
        mv.addObject("user", session.getAttribute("user"));
        mv.addObject("type", type);

        String query = request.getParameter("query");
        if (query == null || query.isEmpty()) {
            mv.addObject("error", "No query");
        } else {
            List<Customer> customers = new ArrayList<Customer>();
            try {
                customers.addAll(new MagasinHelper().getClientsColums(Integer.parseInt(query)));
            } catch (NumberFormatException exception) {

            }

            query = "%".concat(query.concat("%"));
            customers.addAll(new MagasinHelper().getClientsColums(query));

            if (customers.size() == 0) {
                mv.addObject("error", "No user found :/");
            } else {
                mv.addObject("results", customers);
            }
        }
        MagasinHelper helper = new MagasinHelper();
        return mv;
    }

    public ModelAndView achats(HttpServletRequest request,
            HttpServletResponse response) throws Exception {
        sessionInit(request);
        ModelAndView mv = new ModelAndView("achats");
        mv.addObject("user", session.getAttribute("user"));
        mv.addObject("achats", new MagasinHelper().getPurchases(Integer.parseInt(request.getParameter("customerId"))));
        return mv;
    }

    public ModelAndView delete(HttpServletRequest request,
            HttpServletResponse response) throws Exception {
        sessionInit(request);
        String type = request.getParameter("type");
        ModelAndView mv = new ModelAndView("confirm");
        mv.addObject("user", session.getAttribute("user"));
        mv.addObject("type", type);

        switch (type) {
            case "customer":
                new MagasinHelper().deleteCustomer(Integer.parseInt(request.getParameter("customerId")));
                break;
            case "product":
                new MagasinHelper().deleteProduct(Integer.parseInt(request.getParameter("productId")));
                break;
            case "purchase":
                new MagasinHelper().deletePurchase(Integer.parseInt(request.getParameter("orderNum")));
                break;
            case "discount":
                new MagasinHelper().deleteDiscountCode(request.getParameter("discountCode").charAt(0));
                break;
            case "prodCode":
                new MagasinHelper().deleteProductCode(request.getParameter("prodCode"));
                break;
            default:
                mv.addObject("error", "Type not found");
                return mv;
        }

        mv.addObject("confirm", "suppression effectuée");
        return mv;

    }

    private char paramChar(HttpServletRequest request, String name) throws InvalidParameter {
        String param = paramNotNull(request, name);
        if (param.length() > 1) {
            throw new InvalidParameter("Invalid ".concat(name));
        }
        return param.charAt(0);
    }

    private int parseParamInt(HttpServletRequest request, String name) throws InvalidParameter {
        try {
            return Integer.parseInt(request.getParameter(name));
        } catch (Exception e) {
            throw new InvalidParameter("Invalid numer for ".concat(name));
        }
    }

    private int parseParamInt(String param, String name) throws InvalidParameter {
        try {
            return Integer.parseInt(param);
        } catch (Exception e) {
            throw new InvalidParameter("Invalid numer for ".concat(name));
        }
    }

    private BigDecimal parseParamBigDecimal(HttpServletRequest request, String name, int precision) throws InvalidParameter {
        BigDecimal bc;
        try {
            bc = BigDecimal.valueOf(Long.parseLong(request.getParameter("rate")));
        } catch (Exception e) {
            throw new InvalidParameter("Invalid big int for ".concat(name));
        }

        if (bc.precision() > precision) {
            throw new InvalidParameter("Precision for ".concat(name).concat(" shounld be over ".concat(String.valueOf(precision))));
        }
        return bc;
    }

    private Short parseParamShort(HttpServletRequest request, String name) throws InvalidParameter {
        try {
            return Short.valueOf(request.getParameter("rate"));
        } catch (Exception e) {
            throw new InvalidParameter("Invalid short number for ".concat(name));
        }
    }

    private String paramNotNull(HttpServletRequest request, String name) throws InvalidParameter {
        String param = request.getParameter(name);
        if (param == null || param.isEmpty()) {
            throw new InvalidParameter(name.concat(" should not be empty"));
        }
        return param;
    }

    private String paramLength(HttpServletRequest request, String name, int length) throws InvalidParameter {
        String param = request.getParameter(name);
        if (param.length() > length) {
            throw new InvalidParameter(name.concat(" should not be more than ".concat(String.valueOf(length)).concat(" characters")));
        }
        return param;
    }

    private String paramLength(String param, String name, int length) throws InvalidParameter {
        if (param.length() > length) {
            throw new InvalidParameter(name.concat(" should not be more than ".concat(String.valueOf(length)).concat(" characters")));
        }
        return param;
    }

    private String paramAvailable(HttpServletRequest request, String name) throws InvalidParameter {
        String param = request.getParameter(name).toUpperCase();
        if (!param.equals("TRUE") && !param.equals("FALSE")) {
            throw new InvalidParameter("It's true or false for ".concat(name));
        }
        return param;
    }

    private Date paramDate(HttpServletRequest request, String name) throws InvalidParameter {
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
        try {
            return format.parse(request.getParameter(name));
        } catch (Exception e) {
            throw new InvalidParameter("Invalid date for ".concat(name));
        }
    }

    private Object fetchDetailsData(HttpServletRequest request, ModelAndView mv) throws Exception {
        String type = request.getParameter("type");
        Object data = null;

        try {
            switch (type) {
                case "customer":
                    int customerId = parseParamInt(request, "customerId");
                    char discountCode = paramChar(request, "discountCode");
                    String zip = paramLength(paramNotNull(request, "zip"), "zip", 10);
                    String name = paramLength(request, "name", 30);
                    String addressline1 = paramLength(request, "addressline1", 30);
                    String addressline2 = paramLength(request, "addressline2", 30);
                    String city = paramLength(request, "city", 25);
                    String state = paramLength(request, "state", 2);
                    String phone = paramLength(request, "phone", 12);
                    String fax = paramLength(request, "fax", 12);
                    String email = paramLength(request, "email", 40);
                    int creditLimit = parseParamInt(request, "creditLimit");
                    data = new Customer(customerId, discountCode, zip, name, addressline1, addressline2, city, state, phone, fax, email, creditLimit);
                    break;
                case "product":
                    int productId = parseParamInt(request, "productId");
                    int manufacturerId = parseParamInt(paramNotNull(request, "manufacturerId"), "manufacturerId");
                    String productCode = paramLength(paramNotNull(request, "productCode"), "productCode", 2);
                    BigDecimal purchaseCost = parseParamBigDecimal(request, "purchaseCost", 12);
                    int quantityOnHand = parseParamInt(request, "quantityOnHand");
                    BigDecimal markup = parseParamBigDecimal(request, "markup", 12);
                    String available = paramAvailable(request, "available");
                    String description = paramLength(request, "description", 50);
                    data = new Product(productId, manufacturerId, productCode, purchaseCost, quantityOnHand, markup, available, description);
                    break;
                case "purchase":
                    int orderNum = parseParamInt(request, "orderNum");
                    customerId = parseParamInt(paramNotNull(request, "customerId"), "customerId");
                    productId = parseParamInt(paramNotNull(request, "productId"), "productId");
                    Short quantity = parseParamShort(request, "quantity");
                    BigDecimal shippingCost = parseParamBigDecimal(request, "shippingCost", 12);
                    Date salesDate = paramDate(request, "salesDate");
                    Date shippingDate = paramDate(request, "shippingDate");
                    String freightCompany = paramLength(request, "freightCompany", 30);
                    data = new PurchaseOrder(orderNum, customerId, productId, quantity, shippingCost, salesDate, shippingDate, freightCompany);
                    break;
                case "discount":
                    discountCode = paramChar(request, "discountCode");
                    BigDecimal rate = parseParamBigDecimal(request, "rate", 4);
                    data = new DiscountCode(discountCode, rate);
                    break;
                case "prodCode":
                    productCode = paramLength(request, "prodCode", 2);
                    discountCode = paramChar(request, "discountCode");
                    description = paramLength(request, "description", 10);
                    data = new ProductCode(productCode, discountCode, description);
                    break;
                default:
                    mv.addObject("error", "Type not found");
                    return mv;
            }
        } catch (InvalidParameter e) {
            mv.addObject("error", e.getMessage());
        }

        return data;
    }

    public ModelAndView save(HttpServletRequest request,
            HttpServletResponse response) throws Exception {
        sessionInit(request);
        String type = request.getParameter("type");
        ModelAndView mv = new ModelAndView("confirm");
        mv.addObject("user", session.getAttribute("user"));
        mv.addObject("type", type);
        Object data = fetchDetailsData(request, mv);

        if (data != null) {
            try {
                new MagasinHelper().add(data);
                mv.addObject("confirm", "Save completed");
            } catch (ConstraintViolationException e) {
                mv.addObject("error", e.getSQLException().getMessage());
            } catch (Exception e) {
                mv.addObject("error", e.getMessage());
            }
        }
        return mv;
    }

    public ModelAndView update(HttpServletRequest request,
            HttpServletResponse response) throws Exception {
        sessionInit(request);
        String type = request.getParameter("type");
        ModelAndView mv = new ModelAndView("confirm");
        mv.addObject("user", session.getAttribute("user"));
        mv.addObject("type", type);
        Object data = fetchDetailsData(request, mv);

        if (data != null) {
            try {
                new MagasinHelper().update(data);
                mv.addObject("confirm", "Update completed");
            } catch (Exception e) {
                mv.addObject("error", e.getMessage());
            }
        }
        return mv;
    }
}
